/**
 * @file stat_tools.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef STAT_TOOLS_H_INCLUDED
#define STAT_TOOLS_H_INCLUDED

#include <vector>

namespace BPF
{
//! Calcule le factoriel
template<class T>
T factorial(
  T val /**< [in] Entier dont on veut calculer le factoriel*/)
{
	T res(1);
	for (T i = 2; i <= val; ++i)
		res *= i;
	return res;
}

namespace Internal
{
/**
 * Fonction récursive qui génère toutes les combinaisons d'une liste de valeurs.
 * Cette fonction ne doit être utilisée que par BPF::allCombin
 * @param usedList [in] Tableau définissant si chaque valeur est déjà utilisée ou non
 * @param iList    [in] Combinaison de départ
 * @param resultList [in] Combinaison en cours de construction
 * @param resultListOfList [in, out] Liste des combinaisons
 */
template<class I>
void allCombin(
	std::vector<bool> &usedList,
	const I &iList,
	I &resultList,
	std::vector<I> &resultListOfList)
{
	if (iList.size() == resultList.size())
	{
		resultListOfList.push_back(resultList);
		return;
	}

	int index = 0;
	for (typename I::const_iterator iter = iList.begin();
			 iter != iList.end();
			 iter++, index++)
	{
		if (usedList[index] == false)
		{
			usedList[index] = true;
			resultList.push_back(*iter);
			Internal::allCombin(usedList, iList, resultList, resultListOfList);
			resultList.pop_back();
			usedList[index] = false;
		}
	}
}
}

/**
 * Fonction récursive qui génère toutes les combinaisons pour liste de valeurs.
 */
template<class I>
void allCombin(
  const I &iList /** [in] Liste de valeurs dont on veut toutes les combinaisons */,
  std::vector<I> &resultListOfList /** [out] Liste de toutes les combinaisons */)
{
	resultListOfList.clear();
	resultListOfList.reserve(factorial(iList.size()));
	if (iList.empty())
		resultListOfList.push_back(I());

	std::vector<bool> usedList;
	usedList.assign(iList.size(), false);
	I resultList;
	resultList.reserve(iList.size());
	int index = 0;
	for (typename I::const_iterator iter = iList.begin();
			 iter != iList.end();
			 ++iter, ++index)
	{
		usedList[index] = true;
		resultList.push_back(*iter);
		Internal::allCombin(usedList, iList, resultList, resultListOfList);
		resultList.pop_back();
		usedList[index] = false;
	}
}
}

#endif // STAT_TOOLS_H_INCLUDED
