/**
 * @file types.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef TYPES_H_INCLUDED
#define TYPES_H_INCLUDED

#include <boost/unordered_set.hpp>
#include <vector>

namespace BPF
{
typedef boost::unordered_set<int> IntSet;
typedef IntSet::iterator IntSetIterator;
typedef IntSet::const_iterator IntSetConstIterator;
typedef std::vector<IntSet> IntSetVect;
typedef IntSetVect::iterator IntSetVectIterator;
typedef IntSetVect::const_iterator IntSetVectConstIterator;

//! Représente une étape dans le chemin
struct Step
{
	//! Ligne de l'étape
	int lineID;
	//! Station qui termine l'étape
	int statID;
	//! Durée de l'étape, en seconde
	unsigned int duration;
	Step(
    int l /** [in] Ligne de l'étape */,
    int s /** [in] Station qui termine l'étape */,
    unsigned int dur /** [in] Durée de l'étape, en seconde */):
			lineID(l),
			statID(s),
			duration(dur)
	{
	}
};

//! Clef de la map TimeBetweenStationOnLineMap
/**
 * Représente une clef dans la map de type TimeBetweenStationOnLineMap
 * @see TimeBetweenStationOnLineMap
 */
struct LineStationToStationKey
{
	unsigned int lineID;
	unsigned int stationID1;
	unsigned int stationID2;

	LineStationToStationKey(int liID, int sta1ID, int sta2ID):
			lineID(liID),
			stationID1(sta1ID),
			stationID2(sta2ID)
	{
	}

	bool operator == (const LineStationToStationKey &other) const
	{
		return (stationID1 == other.stationID1)
					 && (stationID2 == other.stationID2)
					 && (lineID == other.lineID);
	}
};

//! Hachage de LineStationToStationKey
/**
 * Fonction de hachage associée à LineStationToStationKey.
 * Utilisée pour le TimeBetweenStationOnLineMap.
 * @see LineStationToStationKey
 */
struct hash : public std::unary_function<LineStationToStationKey, std::size_t>
{
	std::size_t operator()(LineStationToStationKey const &p) const
	{
		std::size_t seed = 0;
		boost::hash_combine(seed, p.lineID);
		boost::hash_combine(seed, p.stationID1);
		boost::hash_combine(seed, p.stationID2);
		return seed;
	}
};

//! Map donnant une durée en seconde, pour deux stations sur une ligne.
typedef boost::unordered_map<LineStationToStationKey, int, hash> TimeBetweenStationOnLineMap;
}

#endif // TYPES_H_INCLUDED
