/**
 * @file DVPStationDBLoader.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "DVPStationDBLoader.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include "utf16_codecvt_facet.h"
#include <boost/program_options/detail/utf8_codecvt_facet.hpp>
#include "StationsDB.h"
#include "Station.h"
#include "Line.h"

using std::basic_string;
using std::wifstream;
using std::locale;
using std::wstring;

namespace
{
//! Supprime les espaces en début et fin de chaîne
template<class C>
basic_string<C> trim(const basic_string<C> &inputString)
{
	basic_string<C> output(inputString);
	output.erase(output.find_last_not_of(' ') + 1);
	output.erase(0, output.find_first_not_of(' '));
	return output;
}
}

void DVPStationDBLoader::load(
	StationsDB &stationsDB /**< [out] StationsDB à remplir*/)
{
	StationHandler &stationHandler = stationsDB.getStationHandler();
	LineHandler &lineHandler = stationsDB.getLineHandler();

	stationHandler.clear();
	lineHandler.clear();

	bool isUTF16 = false;
	{
		std::filebuf fileTest;
		fileTest.open(sourceFile_.c_str(), std::ios::in);
		if (fileTest.is_open() == false)
		{
			std::wstringstream errorStream;
			errorStream << L"Impossible d'ouvrir le fichier " << sourceFile_.c_str();
			throw LoadError(errorStream.str());
		}
		unsigned char a, b;
		a = fileTest.sgetc();
		b = fileTest.snextc();
		if (a == 0xFF && b == 0xFE)
			isUTF16 = true;
		//std::wcout << (int)a << " " << (int)b << std::endl;
	}

	wifstream fileStream(sourceFile_.c_str());
	if (fileStream.is_open() == false)
	{
		std::wstringstream errorStream;
		errorStream << L"Impossible d'ouvrir le fichier " << sourceFile_.c_str();
		throw LoadError(errorStream.str());
	}
	if (isUTF16)
		fileStream.imbue(locale(locale(), new utf16_codecvt_facet<wchar_t>()));
	else
	{
    //Pour passer le BOM
    wchar_t BOM;
		fileStream.imbue(locale(locale(),
      new boost::program_options::detail::utf8_codecvt_facet()));
		fileStream >> BOM;
	}

	wstring line;

	SharedLine currentMetroLine;

	while (getline(fileStream, line))
	{
		line = trim(line);

#if !defined(_WIN32) && !defined(_WIN64)
		//LH: Je ne comprend pas pourquoi je dois faire ça... mais c'est comme ca.
		if (isUTF16 == false)
			line.erase(line.size() - 1, 1);
#endif

		//Ligne vide: Les lignes vides sont ignorées
		if (line == L"")
			continue;
		//Si la chaîne est de la forme [*****], c'est une ligne.
		else if ((line.size() > 2) && (line[0] == L'[') && (line[line.size() - 1] == L']'))
		{
			wstring lineName(line, 1, line.size() - 2);
			currentMetroLine = lineHandler.createLine(lineName);
		}
		//Sinon, c'est une station
		else
		{
			if (!currentMetroLine)
				throw LoadError(L"Une station n'a pas de ligne");

			SharedConstStation station = stationHandler.getStation(line);
			if (!station)
				station = stationHandler.createStation(line);
			currentMetroLine->addStation(station);
		}
	}
}
