/**
 * @file Display.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "Display.h"

#include <iostream>
#include <iomanip>
#include <sstream>
#include <iomanip>
#include "PathResult.h"
#include "Station.h"
#include "Line.h"

using std::wcout;
using std::wcin;
using std::wstring;
using std::endl;
using std::setw;
using std::setfill;

namespace
{
wstring formatSecond(int second)
{
	std::wstringstream format;
	format << setw(2) << setfill(L'0') << (second / (60 * 60)) << L':';
	format << setw(2) << setfill(L'0') << ((second / 60) % 60) << L':';
	format << setw(2) << setfill(L'0') << (second % 60);

	return format.str();
}
}

void Display::displayResult(
	const PathResult &result)
{
  if(result.empty())
  {
    wcout << L"Cette requ\u00EAte n'a pas de solution." << endl;
    return;
  }


  if(result.size() == 1)
  {
    if(result.begin()->line == NULL)
    {
      wcout << L"Ce trajet ne comporte qu'une station." << endl;
      return;
    }
  }

  int total = 0;
	wcout << L"Dur\u00E9e total: " << result.getTotalMinuteDuration() << endl;
	for (PathResult::const_iterator iter = result.begin();
			 iter != result.end();
			 iter++)
	{
		const PathResult::Step &step = *iter;

		SharedConstLine line = step.line;
		SharedConstStation endStation =
			line->getEndStationInThisDirection(
				step.station1,
				step.station2);

		if (iter == result.begin())
			wcout << L"Station de d\u00E9part : " << step.station1->getName() << endl;

		total += step.changeDuration;
		wcout << L"Prenez la ligne : [" << line->getName() << L"]"
		<< L" -> direction : " << endStation->getName() << endl;
    if(iter != result.begin())
      wcout << L"  Dur\u00E9e du changement : " << formatSecond(step.changeDuration)
      << L" - Total : " << formatSecond(total) << endl;

		total += step.transportDuration;
		wcout << L"Sortez \u00E0 la station : " << step.station2->getName() << endl
    << L"  Dur\u00E9e du trajet : " << formatSecond(step.transportDuration)
    << L" - Total : " << formatSecond(total) << endl << endl;
	}
}

void Display::queryStepDuration(int &dur)
{
	wcout << L"Entrez le temps entre deux stations (en secondes)" << endl;
	wcin >> dur;
	if(wcin.fail())
	{
    std::wcin.clear();
    std::wstring str;
    std::getline(std::wcin, str);
	}
}

void Display::queryChangeDuration(int &cor)
{
	wcout << L"Entrez le temps pour une correspondance (en secondes)" << endl;
	wcin >> cor;
	if(wcin.fail())
	{
    std::wcin.clear();
    std::wstring str;
    std::getline(std::wcin, str);
	}
}

void Display::queryStartStation(std::wstring &stationName)
{
	wcout << L"Entrez le nom de la station de d\u00E9part" << endl;
	getline(wcin, stationName);
	if(wcin.fail())
    std::wcin.clear();
}

void Display::queryIntermediateStation(std::wstring &stationName)
{
	wcout << L"La station " << stationName << L" n'a pas \u00E9t\u00E9 trouv\u00E9e.";
	wcout << L"Veuillez r\u00E9entrer ce nom plus pr\u00E9cis\u00E9ment" << endl;
	getline(wcin, stationName);
	if(wcin.fail())
    std::wcin.clear();
}

void Display::queryEndStation(std::wstring &stationName)
{
	wcout << L"Entrez le nom de la station d'arriv\u00E9e" << endl;
  getline(wcin, stationName);
	if(wcin.fail())
    std::wcin.clear();
}

void Display::dataBaseLoadError(const std::wstring &mess)
{
	wcout << L"Une erreur est survenue lors du chargement de la base de donn\u00E9e:" << endl;
	wcout << mess << endl;
}
