/**
 * @file Engine.h
 * @author Loc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef ENGINE_H_INCLUDED
#define ENGINE_H_INCLUDED

#include <list>
#include <string>
#include <boost/noncopyable.hpp>
#include "StationsDB.h"
#include "Display.h"

class IStationDBLoader;
class IPathFinder;

//! Rpond  la requte de l'utilisateur
/**
 * A pour tche de rpondre  la requte de l'utilisateur
 */
class Engine : boost::noncopyable
{
	//! Chargeur de base de donne
	boost::shared_ptr<IStationDBLoader> dbLoader_; /*!< Chargeur de base de donne */
	//! Trouveur de chemin
	boost::shared_ptr<IPathFinder> pathFinder_;
	//! Base de donnes
	StationsDB stationDB_;
	//! Interface utilisateur
	Display display_;

	//! Nom de la station de dpart
	std::wstring startStationName_;
	//! Liste des stations intermdiaires
	std::vector<std::wstring> stationNameStepList_;
	//! Nom de la station d'arrive
	std::wstring endStationName_;
	//! true si l'ordre des stations doit tre optimis
	bool pathOptimisation_;
	//! Dure entre deux stations, sur une mme ligne
	int stepDuration_;
	//! Dure d'une correspondance
	int changeDuration_;

public:
	Engine(
		boost::shared_ptr<IStationDBLoader> dbLoader /**< [in] Chargeur de base de donne */,
		boost::shared_ptr<IPathFinder> pathFinder /**< [in] Trouveur de chemin */);

	//! Dfinie la dure entre deux stations, sur une mme ligne
	void userSetStepDuration(int stepDuration);

  //! Dfinie la dure d'une correspondance
	void userSetChangeDuration(int changeDuration);

  //! Dfinie le nom de la station de dpart
	void userSetStartStation(const std::wstring &startStation);

  //! Dfinie le nom de la station d'arrive
	void userSetEndStation(const std::wstring &endStation);

  //! Dfinie la liste des stations intermdiaires
	void userAddIntermediateStations(const std::vector<std::wstring> &stepStationVect);

  //! Signal que l'ordre des stations intermdiaires doit tre optimis
	void userWantPathOptimisation();

  //! Lance le traitement de la requte de l'utilisateur
	void run();
};

#endif // MAIN_H_INCLUDED
