/**
 * @file IStationDBLoader.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef ISTATIONDBFILLER_H_INCLUDED
#define ISTATIONDBFILLER_H_INCLUDED

#include <string>
#include <stdexcept>
#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>

class StationsDB;

//! Interface des class qui chargent la base de donnée.
/**
 * Interface de base des class chargeant la base de donnée.
 * Elle ne fait aucun a priori sur la source de la base
 * (fichier, base de donnée, net, autre...), et permettra
 * donc de changer de source facilement
 */
class IStationDBLoader : boost::noncopyable
{
public:
  //! Exception d'erreur de chargement des données
  class LoadError : public std::runtime_error
  {
    std::wstring message_;

    public:
    LoadError(const std::wstring &message):
      std::runtime_error("Erreur de chargement de la base de donnée de station"),
      message_(message)
    {
    }

    ~LoadError() throw() {}

    const std::wstring& getMessage() const {return message_;}
  };

  //! Pour permettre l'héritage
  virtual ~IStationDBLoader(){}

  /**
   * Remplit le StationsDB depuis une source inconnue.
   * Si le StationsDB n'est pas vide il sera vidé.
   * @param[out] stationDB Base de donnée a remplir.
   */
  virtual void load(StationsDB &stationDB) = 0;
};

#endif // ISTATIONDBFILLER_H_INCLUDED
