/**
 * @file LineHandler.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "LineHandler.h"

#include "Line.h"

using std::make_pair;
using std::wstring;
using boost::shared_ptr;

int LineHandler::NextID = 0;

void LineHandler::addLine(boost::shared_ptr<Line> newLine)
{
	lineMap_.insert(make_pair(newLine->getName(), newLine));
	lineIntMap_.insert(make_pair(newLine->getID(), newLine));
}

shared_ptr<Line> LineHandler::createLine(const std::wstring lineName)
{
	SharedLine newLine = Line::Create(NextID, lineName);
	NextID++;
	addLine(newLine);
	return newLine;
}

LineHandler::SharedConstLine LineHandler::getLine(const std::wstring &name) const
{
	LineMapType::const_iterator lineIter = lineMap_.find(name);
	if (lineIter == lineMap_.end())
		return SharedLine();
	else
		return lineIter->second;
}

LineHandler::SharedConstLine LineHandler::getLine(const int id) const
{
	LineIntMapType::const_iterator lineIter = lineIntMap_.find(id);
	if (lineIter == lineIntMap_.end())
		return SharedLine();
	else
		return lineIter->second;
}

void LineHandler::clear()
{
	lineMap_.clear();
	lineIntMap_.clear();
	NextID = 0;
}
