/**
 * @file LineHandler.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef LINEHANDLER_H_INCLUDED
#define LINEHANDLER_H_INCLUDED

#include <map>
#include <boost/shared_ptr.hpp>
#include <boost/unordered_map.hpp>
#include <boost/noncopyable.hpp>

class Line;

//! Représente une ligne de métro
/**
 * Contient la liste des lignes et permet de les
 * retrouver par ID ou par nom.
 * @see Line
 */
class LineHandler : boost::noncopyable
{
 	static int NextID;

	typedef boost::shared_ptr<const Line> SharedConstLine;
  typedef boost::unordered_map<std::wstring, SharedConstLine> LineMapType;
  typedef boost::unordered_map<int, SharedConstLine> LineIntMapType;
  LineMapType lineMap_;
  LineIntMapType lineIntMap_;

  void addLine(boost::shared_ptr<Line> newLine);

public:
  typedef LineMapType::const_iterator const_iterator;

  boost::shared_ptr<Line> createLine(const std::wstring lineName);

  SharedConstLine getLine(const std::wstring &name) const;

  SharedConstLine getLine(const int id) const;

  void clear();

  std::size_t size() const;
  const_iterator begin() const;
  const_iterator end() const;
};

inline std::size_t LineHandler::size() const
{
	return lineIntMap_.size();
}

inline LineHandler::const_iterator LineHandler::begin() const
{
	return lineMap_.begin();
}

inline LineHandler::const_iterator LineHandler::end() const
{
	return lineMap_.end();
}


#endif // LINEHANDLER_H_INCLUDED
