/**
 * @file NoganePathFinder.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef NOGANE_PATHFINDER_H_INCLUDED
#define NOGANE_PATHFINDER_H_INCLUDED

#include <boost/unordered_map.hpp>
#include <boost/unordered_set.hpp>
#include "IPathFinder.h"
#include "BPF/types.h"

//! Implémente IPathFinder
class NoganePathFinder : public IPathFinder
{
  //! Map donnant le temps entre deux stations sur une même ligne
	BPF::TimeBetweenStationOnLineMap stationStationTimeInLine_;

  //! Map donnant la liste des stations accesibles à partir d'une ligne donnée
	BPF::IntSetVect stationsFromLineMap_;

	//! Map donnant la liste des lignes qui desservissent une station donnée
	BPF::IntSetVect linesFromStationMap_;

	//! Map donnant la liste des "croisements" accessibles à partir d'une ligne donnée.
	//! Un croisement est une station qui est desservie par au moins deux lignes
	BPF::IntSetVect crossingFromLineMap_;

	//! Durée entre deux stations, en seconde
	int stepDuration_;
	//! Durée d'une correspondance, ou d'une étape, en seconde
	int changeLineDuration_;


	//! Recherche le plus rapide chemin entre deux stations
	void findBestPathForTwoStation(
		int stationID,
		int destID,
		int &bestTime,
		std::vector<BPF::Step> &bestWay) const;

  //! Recherche le plus court chemin passant par une liste de station
	void findBestPathForStationList(
		const std::vector<int>&,
		int &bestTime,
		std::vector<BPF::Step> &bestWay) const;

public:
	NoganePathFinder();

  virtual void init(
		const StationsDB &stationDB,
		int stepDuration,
		int changeLineDuration);

  virtual boost::shared_ptr<PathResult> find(
    const StationsDB &stationDB,
    SharedConstStation X,
    const std::vector<SharedConstStation> &stepList,
    SharedConstStation Y,
    bool optimizeStep) const;
};

#endif // NOGANE_PATHFINDER_H_INCLUDED
