/**
 * @file PathResult.h
 * @author Loc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef PATHRESULT_H_INCLUDED
#define PATHRESULT_H_INCLUDED

#include <list>
#include <boost/shared_ptr.hpp>
#include <boost/noncopyable.hpp>

class Station;
class Line;
typedef boost::shared_ptr<const Station> SharedConstStation;
typedef boost::shared_ptr<const Line> SharedConstLine;

//! Chemin rsultant
/**
 * Contient le meilleur chemin trouv, sous forme de liste d'tape (Step).
 */
class PathResult : boost::noncopyable
{
  unsigned int totalSecondDuration_; //!< Dure total du parcours, en seconde */

public:

  //! Reprsente une tape du chemin
  /**
   * Reprsente une tape du chemin.
   * Si les deux Station sont identiques,
   * c'est qu'il s'agit d'un changement de ligne,
   * sur une mme station.
   */
  struct Step
  {
    SharedConstStation station1; //!< Station de dbut de l'tape
    SharedConstStation station2; //!< Station de fin de l'tape
    SharedConstLine line;        //!< Ligne reliant les deux stations
    unsigned int changeDuration; //!< Dure du changement de station, en seconde
    unsigned int transportDuration; //!< Dure du trajet en mtro, en seconde

    Step(
      SharedConstStation sta1,
      SharedConstStation sta2,
      SharedConstLine li,
      unsigned int chaDur,
      unsigned int traDur
    );
  };

private:
  typedef std::list<Step> StepList;

public:
  typedef StepList::iterator iterator;
  typedef StepList::const_iterator const_iterator;

private:
  std::list<Step> stepList_;

public:
  PathResult();

  void addStep(const Step &newStep);

  unsigned int getTotalMinuteDuration() const;

  size_t size() const;

  bool empty() const;

  const_iterator begin() const;
  const_iterator end() const;
};

inline unsigned int PathResult::getTotalMinuteDuration() const
{
	return totalSecondDuration_;
}

inline PathResult::const_iterator PathResult::begin() const
{
	return stepList_.begin();
}

inline PathResult::const_iterator PathResult::end() const
{
	return stepList_.end();
}

inline size_t PathResult::size() const
{
  return stepList_.size();
}

inline bool PathResult::empty() const
{
  return stepList_.empty();
}

#endif // PATHRESULT_H_INCLUDED

