/**
 * @file StationsDB.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef STATIONDB_H_INCLUDED
#define STATIONDB_H_INCLUDED

#include "StationHandler.h"
#include "LineHandler.h"
#include <boost/noncopyable.hpp>

//! Base de donnée du plan de métro
/**
 * Contient à la fois le gestionnaire de ligne, et de station.
 * @see StationHandler
 * @see LineHandler
 */
class StationsDB : boost::noncopyable
{
	StationHandler stationHandler_;
	LineHandler lineHandler_;

public:
	StationHandler& getStationHandler();
	const StationHandler& getStationHandler() const;
	LineHandler& getLineHandler();
	const LineHandler& getLineHandler() const;
};


inline StationHandler& StationsDB::getStationHandler()
{
	return stationHandler_;
}

inline const StationHandler& StationsDB::getStationHandler() const
{
	return stationHandler_;
}

inline LineHandler& StationsDB::getLineHandler()
{
	return lineHandler_;
}

inline const LineHandler& StationsDB::getLineHandler() const
{
	return lineHandler_;
}

#endif // STATIONDB_H_INCLUDED
