/**
 * @file main.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include <iostream>
#include <locale>
#include <boost/program_options.hpp>
#include <boost/make_shared.hpp>
#include "Engine.h"
#include "DVPStationDBLoader.h"
#include "NoganePathFinder.h"
#include "Tester.h"

#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#else
#include <boost/program_options/detail/utf8_codecvt_facet.hpp>
#endif

using std::string;
using std::wstring;
using std::vector;
using std::wcout;
using std::cout;
using std::endl;
namespace po = boost::program_options;

#if defined(_WIN32) || defined(_WIN64)
int main(int, char *[])
{
	//Pour que wcin et wcout convertissent OEM <-> Unicode
	setlocale(LC_ALL, ".OCP");
	//C'est le moyen le plus simple que j'ai trouvé
	//pour convertir la ligne de commande de l'OEM vers l'Unicode:
	int argc;
	LPWSTR *argv = CommandLineToArgvW(GetCommandLineW(), &argc);
#else
int main(int argc, char *argv[])
{
	//Pour que program_option convertisse la ligne de commande
	//de l'UTF-8 vers l'Unicode
	std::codecvt<wchar_t, char, mbstate_t>* facet =
		new boost::program_options::detail::utf8_codecvt_facet;
	std::locale::global(std::locale(std::locale(), facet));
	//Pour que wcin et wcout convertissent UTF-8 <-> Unicode
	setlocale(LC_ALL, "fr_FR.UTF-8");
#endif

	try
	{
		//Déclaration des options.
		//program_option ne semble pas gérer les wchar à ce niveau,
		//(et puis surtout j'ai pas le temps d'approfondir)
		//donc le plus simple est encore de supprimer les accents.
		po::options_description desc("Options permisent");
		desc.add_options()
		("h,h", "Aide")
		("v,v", "Copyright")
		("f,f", po::value<string>(), "Fichier de donnee (obligatoire)")
		("d,d", po::wvalue<unsigned int>(), "Interval entre deux stations (secondes)")
		("c,c", po::wvalue<unsigned int>(), "Temps d'attente pour une correspondance (secondes)")
		("x,x", po::wvalue<wstring>(), "Nom de la station de depart")
		("y,y", po::wvalue<wstring>(), "Nom de la station d'arrivee")
		("i,i", po::wvalue<vector<wstring> >(), "Nom d'une station intermediaire")
		("o,o", "Optimisation de l'ordre des stations intermediaires")
		("t,t", "Test");

		po::variables_map vm;
		try
		{
			po::store(po::parse_command_line(argc, argv, desc), vm);
		}
		catch (std::exception &)
		{
			cout << desc << endl;
			return 0;
		}

		po::notify(vm);

		if (vm.count("t"))
		{
			Tester test;
			test.run();
		}

		if (vm.count("h"))
		{
			cout << desc << endl;
			return 0;
		}
		if (vm.count("v"))
		{
			wcout << L"Copyright \u00A92009 Lo\u00EFc HAMOT. Tous droits r\u00E9serv\u00E9s Lo\u00EFc HAMOT." << endl
			<< L"Les sources qui constituent ce projet defi_cpp_4 "
			<< L"de m\u00EAme que la documentation associée sont la propri\u00E9t\u00E9 de leur auteur." << endl
			<< L"Je donne mon accord au site developpez.com "
			<< L"pour l'utilisation de tout ou partie des sources "
			<< L"et de la documentation de ce projet dans les pages developpez.com" << endl;
			return 0;
		}
		if (vm.count("f") == 0)
		{
			cout << desc << endl;
			return 0;
		}

		Engine engine(
			boost::make_shared<DVPStationDBLoader>(vm["f"].as<string>()),
			boost::make_shared<NoganePathFinder>());

		if (vm.count("d"))
			engine.userSetStepDuration(vm["d"].as<unsigned int>());
		if (vm.count("c"))
			engine.userSetChangeDuration(vm["c"].as<unsigned int>());
		if (vm.count("x"))
			engine.userSetStartStation(vm["x"].as<wstring>());
		if (vm.count("y"))
			engine.userSetEndStation(vm["y"].as<wstring>());
		if (vm.count("i"))
			engine.userAddIntermediateStations(vm["i"].as<vector<wstring> >());
		if (vm.count("o"))
			engine.userWantPathOptimisation();

		engine.run();
	}
	catch (std::exception &ex)
	{
		wcout << L"Une exception n'as pas été gérée : " << typeid(ex).name() << endl;
		return 1;
	}

	return 0;
}
